package cn.com.ty.lift.business.maintenance.controller;

import cn.com.ty.lift.business.common.CountPage;
import cn.com.ty.lift.business.maintenance.dao.entity.MaintenanceOption;
import cn.com.ty.lift.business.maintenance.dao.entity.MaintenanceRecord;
import cn.com.ty.lift.business.maintenance.dao.entity.MtRecordImg;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MtPlanRequest;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MtRecordRequest;
import cn.com.ty.lift.business.maintenance.dao.entity.model.response.MtOptionTree;
import cn.com.ty.lift.business.maintenance.dao.entity.model.response.MtPlanResponse;
import cn.com.ty.lift.business.maintenance.dao.entity.model.response.MtRecordResponse;
import cn.com.ty.lift.business.maintenance.service.*;
import cn.com.ty.lift.business.project.dao.entity.ProjectLiftRelevance;
import cn.com.ty.lift.business.project.service.ProjectLiftRelevanceService;
import cn.com.ty.lift.common.utils.Judge;
import cn.com.ty.lift.common.utils.Judger;
import cn.com.xwy.boot.web.dto.RestResponse;
import cn.hutool.core.collection.IterUtil;
import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.google.common.base.Splitter;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.validation.Valid;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.LocalTime;
import java.util.*;
import java.util.stream.Collectors;

/**
　* <p>
  * 维保管理-日常保养
  * <p>
  * @author wcz
　* @since 2020/1/17 17:24
　*/
@Slf4j
@AllArgsConstructor
@RestController
@RequestMapping("maintenance/record")
public class MaintenanceRecordController {

    private MaintenancePlanService maintenancePlanService;
    private MaintenanceRecordService maintenanceRecordService;
    private MtRecordCostService mtRecordCostService;
    private MaintenanceOptionService maintenanceOptionService;
    private MtRecordImgService mtRecordImgService;
    private ProjectLiftRelevanceService projectLiftRelevanceService;

    //法规维保间隔15天
    private final int maintenance_interval = 15;
    /**
     * 根据条件分页查询维保-日常保养
     * @param request MtRecordRequest
     * @return RestResponse
     */
    @PostMapping("pageRecord")
    @Judger(fields = {"mtCompanyId"})
    public RestResponse pageRecord(@RequestBody MtRecordRequest request){
        IPage<MtRecordResponse> pages = maintenanceRecordService.pageRecordByCondition(request);
        long count = maintenanceRecordService.countRecordByCondition(request);
        CountPage<MtRecordResponse> countPage = new CountPage<>(count,pages);
        return RestResponse.success(countPage);
    }

    /**
     * 根据条件分页查询超期的维保计划
     * @param request MtPlanRequest
     * @return RestResponse
     */
    @PostMapping("pagePlan")
    @Judger(fields = {"mtCompanyId"})
    public RestResponse pagePlan(@RequestBody MtPlanRequest request){
        IPage<MtPlanResponse> pages = maintenancePlanService.pagePlanByCondition(request);
        Judge.notNull(pages);
        List<MtPlanResponse> records = pages.getRecords();
        if(IterUtil.isNotEmpty(records)){
            List<ProjectLiftRelevance> relevances = projectLiftRelevanceService.listByMtCompany(request.getMtCompanyId());
            records.forEach(record -> {
                Optional<ProjectLiftRelevance> liftRelevance = relevances
                        .stream()
                        .filter(r -> (r.getLiftId().equals(record.getLiftId()) && r.getProjectId().equals(record.getProjectId())))
                        .findFirst();
                liftRelevance.ifPresent(lr -> {
                    log.info("PlanDate: {}",record.getPlanDate());
                    LocalDate deadline = record.getPlanDate().plusDays(maintenance_interval - lr.getPlanInterval());
                    log.info("deadline: {}",deadline);
                    //如果超过法规15天，法规超期LocalDate.now() < deadline
                    LocalDateTime max = LocalDateTime.of(deadline,LocalTime.MAX);
                    log.info("maxtime: {}",max);
                    record.setOverdue(LocalDateTime.now().isBefore(max) ? 1 : 2);
                    log.info("overdue: {}",record.getOverdue());
                });
            });
        }
        long count = maintenancePlanService.countPlanByCondition(request);
        CountPage<MtPlanResponse> countPage = new CountPage<>(count,pages);
        return RestResponse.success(countPage);
    }

    /**
     * 根据id查找,带关联信息
     * @param request MtRecordRequest
     * @return RestResponse
     */
    @PostMapping("one")
    @Judger(fields = {"id"})
    public RestResponse one(@RequestBody MtRecordRequest request){
        MtRecordResponse entity = maintenanceRecordService.findByIdWithInfo(request);
        Judge.notNull(entity);
        /**
         * 维保项id:0/1/2, 0:无需保养，1：已保养，2：需要更换
         * 保养项目  用键值对的方式存储，如(1:0,2:1,3:1...)
         */
        String maintenanceOption = entity.getMaintenanceOption();
        if(StrUtil.isNotEmpty(maintenanceOption)){
            Map<String, String> opts = Splitter.on(",").withKeyValueSeparator(":").split(maintenanceOption);
            Map<Long,Integer> optMap = new HashMap<>();
            opts.forEach((key, value) -> {
                optMap.put(Long.valueOf(key),Integer.valueOf(value));
            });
            Collection<MaintenanceOption> mtOptions = maintenanceOptionService.listByIds(optMap.keySet());
            mtOptions.forEach(option -> {
                option.setOpt(optMap.get(option.getId()));
            });
            entity.setMtOptions(mtOptions);
        }
        Long id = request.getId();
        //查询费用项
        entity.setMtRecordCosts(mtRecordCostService.listByRecord(id));
        //查询图片
        entity.setMtRecordImgs(mtRecordImgService.listByRecord(id));
        return RestResponse.success(entity);
    }

    /**
     * 维保项 - 树形
     * @param request MtRecordRequest
     * @return RestResponse
     */
    @PostMapping("treeOptions")
    @Judger(fields = {"mtCompanyId","type","liftType"})
    public RestResponse treeOptions(@RequestBody MtRecordRequest request){
        Integer type = request.getType();
        Integer liftType = request.getLiftType();
        Long mtCompanyId = request.getMtCompanyId();
        List<MaintenanceOption> optionList = maintenanceOptionService.listByTypeAndLiftType(type,liftType,mtCompanyId);
        Judge.notNull(optionList);
        //按照sort groupby
        Map<Integer, List<MaintenanceOption>> mtOptionMap = optionList.stream().collect(Collectors.groupingBy(MaintenanceOption::getSort));
        Judge.notNull(mtOptionMap);
        List<MtOptionTree> mtOptionTrees = new ArrayList<>();
        mtOptionMap.forEach((sort,options) ->{
            mtOptionTrees.add(new MtOptionTree(sort,options));
        });
        return RestResponse.success(mtOptionTrees);
    }

    /**
     * 维保项list
     * @param request MtRecordRequest
     * @return RestResponse
     */
    @PostMapping("listOptions")
    @Judger(fields = {"mtCompanyId","type","liftType"})
    public RestResponse listOptions(@RequestBody MtRecordRequest request){
        Integer type = request.getType();
        Integer liftType = request.getLiftType();
        Long mtCompanyId = request.getMtCompanyId();
        List<MaintenanceOption> optionList = maintenanceOptionService.listByTypeAndLiftType(type,liftType,mtCompanyId);
        return RestResponse.success(optionList);
    }
    /**
     * 点击填写保养单，带收费项，新增record
     * @param entity MaintenanceRecord
     * @return RestResponse
     */
    @PostMapping("tofill")
    public RestResponse tofill(@Valid @RequestBody MaintenanceRecord entity){
        //补录
        entity.setIsRepair(1);
        maintenanceRecordService.tofill(entity);
        return RestResponse.success(entity.getId());
    }

    /**
     * 填写保养单-确认，带维保项和现场图片 ，修改record
     * @param request MtRecordRequest
     * @return RestResponse
     */
    @PostMapping("fill")
    @Judger(fields = {"id","maintenanceOption","imgs"})
    public RestResponse fill(@RequestBody MtRecordRequest request){
        Long id = request.getId();
        String maintenanceOption = request.getMaintenanceOption();
        List<String> imgs = request.getImgs();
        List<MtRecordImg> mtRecordImgs = new ArrayList<>();
        for (int i = 0; i < imgs.size(); i++) {
            MtRecordImg mri = new MtRecordImg();
            mri.setMtRecordId(id);
            mri.setImgUrl(imgs.get(i));
            mri.setSort(i);
            mtRecordImgs.add(mri);
        }
        MaintenanceRecord entity = maintenanceRecordService.getById(id);
        Judge.notNull(entity);
        entity.setMaintenanceOption(maintenanceOption);
        boolean result = maintenanceRecordService.fill(entity,mtRecordImgs);
        return RestResponse.success(result);
    }
    /**
     * 补录工单，带收费项，修改record
     * @param entity MaintenanceRecord
     * @return RestResponse
     */
    @PostMapping("workOrder")
    public RestResponse workOrder(@Valid @RequestBody MaintenanceRecord entity){
        boolean result = maintenanceRecordService.tofill(entity);
        return RestResponse.success(result);
    }
}
