package cn.com.ty.lift.common.utils;

import cn.hutool.core.util.ObjectUtil;

import java.lang.reflect.Field;

/**
 * @author wcz
 * @Description: 条件判断，如果符合条件，返回异常
 * @date 2020/1/9 8:46
 */
public interface Judge {

    String INVALID_IDENTIFIER = "标识ID无效";
    String MUST_GT0 = "数值必须大于0";
    String MUST_LT0 = "数值必须小于0";
    String MUST_EQ0 = "数值必须等于0";
    String MUST_NGT0 = "数值必须大于或等于0";
    String MUST_NLT0 = "数值必须小于或等于0";
    String MUST_NEQ0 = "数值必须不等于0";
    String MISSING_INCORRECT_DATA = "缺少数据或数据有误";
    String INVALID_DATA_CONDITIONS = "数据无效或条件不成立";

    /**
     * id == null || id <= 0 throw new JudgeException(message)
     *
     * @param id
     */
    static void id(Long id, String message) {
        notNull(id, message);
        if (id.longValue() <= 0) {
            throw new JudgeException(message);
        }
    }

    /**
     * id == null || id <= 0 throw new JudgeException(INVALID_IDENTIFIER)
     *
     * @param ids
     */
    static void id(Long... ids) {
        notNull(ids, INVALID_IDENTIFIER);
        for (Long id : ids) {
            if (null == id || id.longValue() <= 0) {
                throw new JudgeException(INVALID_IDENTIFIER);
            }
        }
    }

    /**
     * value == null || value <= 0 throw new JudgeException(message);
     *
     * @param value
     * @param message
     */
    static void gt0(Number value, String message) {
        if (null == value) {
            throw new JudgeException(message);
        }
        if (value instanceof Long && value.longValue() <= 0) {
            throw new JudgeException(message);
        }
        if (value instanceof Integer && value.intValue() <= 0) {
            throw new JudgeException(message);
        }
        if (value instanceof Float && value.floatValue() <= 0) {
            throw new JudgeException(message);
        }
        if (value instanceof Double && value.doubleValue() <= 0) {
            throw new JudgeException(message);
        }
    }

    /**
     * value == null || value <= 0 throw new JudgeException(message);
     *
     * @param values
     */
    static void gt0(Number... values) {
        notNull(values, MUST_GT0);
        for (Number value : values) {
            gt0(value, MUST_GT0);
        }
    }

    /**
     * value == null || value >= 0 throw new JudgeException(message);
     *
     * @param value
     * @param message
     */
    static void lt0(Number value, String message) {
        if (null == value) {
            throw new JudgeException(message);
        }
        if (value instanceof Long && value.longValue() >= 0) {
            throw new JudgeException(message);
        }
        if (value instanceof Integer && value.intValue() >= 0) {
            throw new JudgeException(message);
        }
        if (value instanceof Float && value.floatValue() >= 0) {
            throw new JudgeException(message);
        }
        if (value instanceof Double && value.doubleValue() >= 0) {
            throw new JudgeException(message);
        }
    }

    /**
     * value == null || value >= 0 throw new JudgeException(message);
     *
     * @param values
     */
    static void lt0(Number... values) {
        notNull(values, MUST_LT0);
        for (Number value : values) {
            lt0(value, MUST_LT0);
        }
    }

    /**
     * value == null || value != 0 throw new JudgeException(message);
     *
     * @param value
     * @param message
     */
    static void eq0(Number value, String message) {
        if (null == value) {
            throw new JudgeException(message);
        }
        if (value instanceof Long && value.longValue() != 0) {
            throw new JudgeException(message);
        }
        if (value instanceof Integer && value.intValue() != 0) {
            throw new JudgeException(message);
        }
        if (value instanceof Float && value.floatValue() != 0) {
            throw new JudgeException(message);
        }
        if (value instanceof Double && value.doubleValue() != 0) {
            throw new JudgeException(message);
        }
    }

    static void eq0(Number... values) {
        notNull(values, MUST_EQ0);
        for (Number value : values) {
            eq0(value, MUST_EQ0);
        }
    }

    /**
     * value == null || value < 0 throw new JudgeException(message)
     *
     * @param value
     * @param message
     */
    static void ngt0(Number value, String message) {
        if (null == value) {
            throw new JudgeException(message);
        }
        if (value instanceof Long && value.longValue() < 0) {
            throw new JudgeException(message);
        }
        if (value instanceof Integer && value.intValue() < 0) {
            throw new JudgeException(message);
        }
        if (value instanceof Float && value.floatValue() < 0) {
            throw new JudgeException(message);
        }
        if (value instanceof Double && value.doubleValue() < 0) {
            throw new JudgeException(message);
        }
    }

    /**
     * value == null || value < 0 throw new JudgeException(message);
     *
     * @param values
     */
    static void ngt0(Number... values) {
        notNull(values, MUST_NGT0);
        for (Number value : values) {
            ngt0(value, MUST_NGT0);
        }
    }

    /**
     * value == null || value > 0 throw new JudgeException(message);
     *
     * @param value
     * @param message
     */
    static void nlt0(Number value, String message) {
        if (null == value) {
            throw new JudgeException(message);
        }
        if (value instanceof Long && value.longValue() > 0) {
            throw new JudgeException(message);
        }
        if (value instanceof Integer && value.intValue() > 0) {
            throw new JudgeException(message);
        }
        if (value instanceof Float && value.floatValue() > 0) {
            throw new JudgeException(message);
        }
        if (value instanceof Double && value.doubleValue() > 0) {
            throw new JudgeException(message);
        }
    }

    /**
     * value == null || value > 0 throw new JudgeException(message);
     *
     * @param values
     */
    static void nlt0(Number... values) {
        notNull(values, MUST_NLT0);
        for (Number value : values) {
            nlt0(value, MUST_NLT0);
        }
    }

    /**
     * value == null || value == 0 throw new JudgeException(message);
     *
     * @param value
     * @param message
     */
    static void neq0(Number value, String message) {
        if (null == value) {
            throw new JudgeException(message);
        }
        if (value instanceof Long && value.longValue() == 0) {
            throw new JudgeException(message);
        }
        if (value instanceof Integer && value.intValue() == 0) {
            throw new JudgeException(message);
        }
        if (value instanceof Float && value.floatValue() == 0) {
            throw new JudgeException(message);
        }
        if (value instanceof Double && value.doubleValue() == 0) {
            throw new JudgeException(message);
        }
    }

    /**
     * value == null || value != 0 throw new JudgeException(message);
     *
     * @param values
     */
    static void neq0(Number... values) {
        notNull(values, MUST_NEQ0);
        for (Number value : values) {
            neq0(value, MUST_NEQ0);
        }
    }

    /**
     * 判断对象object是否为空，如果不为空,直接抛出异常，返回message
     *
     * @param object
     * @param message
     */
    static void isNull(Object object, String message) {
        if (ObjectUtil.isNotEmpty(object)) {
            throw new JudgeException(message);
        }
    }

    /**
     * 判断对象object是否为空，如果不为空,直接抛出异常，返回默认消息
     *
     * @param object
     */
    static void isNull(Object object) {
        isNull(object, MISSING_INCORRECT_DATA);
    }

    /**
     * 判断对象object是否为空，如果为真,直接抛出异常，返回message
     *
     * @param object
     * @param message
     */
    static void notNull(Object object, String message) {
        if (ObjectUtil.isEmpty(object)) {
            throw new JudgeException(message);
        }
    }

    /**
     * 判断对象object是否为空，如果为真,直接抛出异常，返回默认消息
     *
     * @param object
     */
    static void notNull(Object object) {
        notNull(object, MISSING_INCORRECT_DATA);
    }

    /**
     * 判断表达式是否为真，如果为假，直接抛出异常，返回message
     *
     * @param expression
     * @param message
     */
    static void isTrue(boolean expression, String message) {
        if (!expression) {
            throw new JudgeException(message);
        }
    }

    /**
     * 判断表达式是否为真，如果为假，直接抛出异常，返回
     *
     * @param expression
     */
    static void isTrue(boolean expression) {
        isTrue(expression, INVALID_DATA_CONDITIONS);
    }

    /**
     * 判断表达式是否为真，如果为真，直接抛出异常，返回message
     *
     * @param expression
     * @param message
     */
    static void notTrue(boolean expression, String message) {
        if (expression) {
            throw new JudgeException(message);
        }
    }

    /**
     * 判断表达式是否为真，如果为真，直接抛出异常，返回
     *
     * @param expression
     */
    static void notTrue(boolean expression) {
        notTrue(expression, INVALID_DATA_CONDITIONS);
    }

    /**
     * 根据属性名检查object中是否有效值,
     * 数值型的值，必须!null && > 0 ,否则 throw new JudgeException(message)
     * message：对应object中属性用@Judger标识中的message
     * @param object 校验的对象
     * @param fields 需要校验的属性名
     */
    static void judgement(Object object, String... fields) {
        notNull(object);
        notNull(fields);
        Class<?> objectClass = object.getClass();
        notNull(objectClass);
        try {
            for (String field : fields) {
                Field declaredField = objectClass.getDeclaredField(field);
                if (null != declaredField) {
                    declaredField.setAccessible(true);
                    Judger judger = declaredField.getDeclaredAnnotation(Judger.class);
                    Class<?> fieldClass = declaredField.getType();
                    Object value = declaredField.get(object);
                    if (Number.class.isAssignableFrom(fieldClass)) {
                        gt0((Number) value, null == judger ? MUST_GT0 : judger.message());
                    } else {
                        notNull(value, null == judger ? MISSING_INCORRECT_DATA : judger.message());
                    }
                }
            }
        } catch (Exception e) {
            throw new JudgeException(e.getMessage());
        }
    }
}
